#!/bin/sh

outcar=""
xmol="no"
moviemol="no"
rms="no"
evs="no"
toten="no"
ekin="no"
etotal="no"
maxforce="no"
help="yes"
zipcom="cat"

options=$*

show_help()
{
     echo ""
     echo " Usage:"
     echo ""
     echo "  $0 <options> <input file name>"
     echo ""
     echo " Synopsis:"
     echo "    Extracts the geometries and the energies from the"
     echo "    OUTCAR output file of VASP and displays them on screen."
     echo "    This screen output can be redirected to any file."
     echo ""
     echo "    If no input filename is give, the default name is \"OUTCAR\""
     echo ""
     echo ""
     echo "  Available options:"
     echo "    (all options are case INSENSITIVE)"
     echo ""
     echo "   -m   :  Extract the geom. in MOVIEMOL format for visualization."
     echo "   -x   :  Extract the geom. in XMOL format for visualization."
     echo "   -r   :  Extract the geom. in KS (our KS code) format for RMS."
     echo "   -e   :  Extract the Eigenvalue spectrum. of all iterations."
     echo "   -b   :  Extract the Energies (TOTEN: binding energies) of all iterations."
     echo "   -k   :  Extract the Kinetic  Energy (EKIN) and temperatures of all iterations."
     echo "   -t   :  Extract the Total Energies (ETOTAL) of all iterations."
     echo "   -f   :  Show the Maximum Force after the last iteration."
     echo "   -bz2 :  Extract the data from a BZIPed file."
     echo "   -gz  :  Extract the data from a GZIPed file."
     echo "   -o <input filename>:  Change the input file to this. This is"
     echo "                         option normally not needed since even"
     echo "                         without this option (i.e. -o) the input"
     echo "                         new filename will be used."

     echo ""
}

set_value()
{
   moviemol=no
   xmol=no
   rms=no
   evs=no
   toten=no
   ekin=no
   etotal=no
   maxforce=no
   help=no
   case $1 in
      moviemol )  moviemol=yes; return;;
      xmol )      xmol=yes; return;;
      rms )       rms=yes; return;;
      evs )       evs=yes; return;;
      toten )     toten=yes; return;;
      ekin )      ekin=yes; return;;
      etotal )    etotal=yes; return;;
      maxforce )  maxforce=yes; return;;
      help )      help=yes; return;;
   esac
}

if [ $# -gt 0 ] ; then
   while [ $# -gt 0 ]
   do
      opt=$1
      case ${opt} in
         -m | -M ) set_value moviemol;;

         -x | -X ) set_value xmol;;

         -r | -R ) set_value rms;;

         -e | -E ) set_value evs;;

         -b | -B ) set_value toten;;

         -k | -K ) set_value ekin;;

         -t | -T ) set_value etotal;;

         -f | -F ) set_value maxforce;;

         -bz2 | -BZ2 | -Bz2 | -bZ2 ) zipcom="bunzip2 -c";;

         -gz | -GZ| -Gz| -gZ ) zipcom="gunzip -c";;

         -o | -O ) shift; outcar="$1";;
         -h ) show_help; exit;;
          * ) outcar="$1"
      esac
      shift
   done
fi


if [ "${help}" = "yes" ] ; then
   show_help
   exit
fi

if [ "${outcar}" = "" ] ; then
   if [ "${zipcom}" = "bunzip2 -c" ] ; then
      outcar=OUTCAR.bz2
   elif [ "${zipcom}" = "gunzip -c" ] ; then
      outcar=OUTCAR.gz
   else
      outcar=OUTCAR
   fi
fi


if [ ! -f "${outcar}" ] ; then
   echo ""
   echo "Error... File \"${outcar}\" does not exists..."
   echo ""
   exit
fi


natoms="`${zipcom} ${outcar}|head -1000|grep \"  NIONS\" |head -1 | awk '{print $12}'`"
nbands="`${zipcom} ${outcar}|head -1000|grep \"  NBANDS\"|head -1 | awk '{print $9}'`"



#
#  Check for MOVIEMOL option...
#
if [ "${moviemol}" = "yes" ] ; then
   nlines=`\expr ${natoms} + 2`
   nframes=`${zipcom} ${outcar} | grep "POSITION     " | wc -l | awk '{print $1}'`
   # if [ ${nframes} -gt 10000 ] ; then
   #    nframes=10000
   # fi
   echo "${nframes}"
   ${zipcom} ${outcar}                                              \
       |  grep -A ${nlines} "POSITION     "                         \
       |  grep -v "\-\-"                                            \
       |  awk '{printf("%9s %9s %9s   11\n",$1,$2,$3)}'             \
       |  sed -e"s/^ POSITION TOTAL-FORCE (eV\/Angst)   11/${natoms}/"
fi


#
#  Check for XMOL option...
#
if [ "${xmol}" = "yes" ] ; then
   
   nlines=`\expr ${natoms} + 2`
   ions_per_type=`${zipcom} ${outcar}|head -2000 | grep "ions per type"`
   ntype=`echo "${ions_per_type}" | wc -w | awk '{print $1-4}'`
   symbols=`${zipcom} ${outcar}|head -2000|grep POTCAR|head -${ntype}|awk '{printf("%s ",$3)}'`
   for ((i=1; i<=ntype; i++))
   do
      atomsymbol[i]=`echo "${symbols}" | awk -v n=${i} '{print $n}'`
   done

   for ((i=1; i<=ntype; i++))
   do
      ((j=4+i))
      atomtype[i]=`echo ${ions_per_type} | awk -v n=${j} '{print $n}'`;
   done

   ((itype=1))
   ((atm_count=0))
   ((iatm_begin=0))
   ((iatm_end=atomtype[itype]))
   for ((iatm=1; iatm<=natoms; iatm++))
   do
      ((atm_count=atm_count+1))
      if [ ${iatm} -gt ${iatm_begin} -a ${iatm} -le ${iatm_end} ] ; then
        atom[iatm]=${atomsymbol[itype]}
      fi


      if [ ${atm_count} -eq ${atomtype[itype]} ] ; then
        ((atm_count=0))
        ((itype=itype+1))
        ((iatm_begin=iatm_end))
        ((iatm_end+=atomtype[itype]))
      fi
   done

   symbol_file="symbol_file"
   printf "" > symbol_file
   for ((iatm=1; iatm<=natoms; iatm++))
   do
      echo "${atom[iatm]}"  >> ${symbol_file}
   done


   ${zipcom} ${outcar}                                              \
       |  grep -A ${nlines} "POSITION     "                         \
       |  grep -v "\-\-\-\-\-"                                      \
       |  awk '{printf("Na %9s %9s %9s\n",$1,$2,$3)}'               \
       |  sed -e"s/^Na        --/${natoms}/"                        \
       |  sed -e"s/^Na  POSITION TOTAL-FORCE (eV\/Angst)//"         \
       |  awk -v symbol_file=${symbol_file} -v natoms=${natoms} '
                BEGIN {
                   for (iatm=1; iatm<=natoms; iatm++) {
                     getline atom[$iatm] < symbol_file;
                     symbol[iatm]=atom[$iatm]
                   }
                }
                {
                  if ($1=="Na")
                    {
                      jatm++
                      printf("%2s  %9s  %9s  %9s\n", symbol[jatm],$2,$3,$4)
                    }
                  else if ($1==natom)
                    {
                      jatm=0
                      printf("%d\n\n", natoms);
                    }
                }
          '

   /bin/rm  -f  ${symbol_file}

fi


#
#  Check for RMS option...
#
if [ "${rms}" = "yes" ] ; then
   nlines=`\expr ${natoms} + 2`
   ${zipcom} ${outcar}                                              \
       |  grep -A ${nlines} "POSITION     "                         \
       |  grep -v "\-\-"                                            \
       |  awk '{printf("1 %9s %9s %9s 1\n",$1,$2,$3)}'      \
       |  sed -e"s/^1  POSITION TOTAL-FORCE (eV\/Angst) 1/1/"
fi


#
#  Check for EIGENVALUE option...
#
if [ "${evs}" = "yes" ] ; then
   nlines=`\expr ${nbands} + 0`
   ${zipcom} ${outcar}                                              \
       |  grep -A ${nlines} "^  band No\.  band energies     occupation"             \
       |  grep -v "\-\-"                                            \
       |  sed -e"s/    .0000/   0.0000/g"                           \
       |  sed -e"s/     //g"                                        \
       |  sed -e"s/    2/ 2/g"                                      \
       |  sed -e"s/    1/ 1/g"                                      \
       |  sed -e"s/    0/ 0/g"                                      \
       |  sed -e"s/  -/-/g"                                         \
       |  grep -v "band"                                            \
       |  sed -e"s/\.00000//g"
fi


#
#  Check for TOTEN (Total Binding Energy) option...
#
if [ "${toten}" = "yes" ] ; then
   # nlines=`\expr ${nbands} + 2`
   ${zipcom} ${outcar} | grep "  free  energy   TOTEN  =" | grep eV | awk '{print $5}'
fi


#
#  Check for EKIN (Kinetic Energy and Temperature) option...
#
if [ "${ekin}" = "yes" ] ; then
   # nlines=`\expr ${nbands} + 2`
   ${zipcom} ${outcar}|grep "  kinetic Energy EKIN   =  " |awk '{printf(" %s  %6.2f\n",$5,$7)}'
fi


#
#  Check for ETOTAL (Total Energy = P.E. + K.E) option...
#
if [ "${etotal}" = "yes" ] ; then
   # nlines=`\expr ${nbands} + 2`
   ${zipcom} ${outcar} | grep "  total energy   ETOTAL =" | awk '{print $5}'
fi


#
#  Check for MAX FORCE option...
#
if [ "${maxforce}" = "yes" ] ; then
   printf "\n   Maximum FORCE is  :  "
   nlines=`\expr ${natoms} + 2`

   ${zipcom} ${outcar}                                \
       |  grep "TOTAL-FORCE" -A ${nlines}             \
       |  grep -v "\-\-\-"                            \
       |  tail -${natoms}                             \
       |  awk -v v=1 '{printf("%9s  ATOM=%d x\n%9s  ATOM=%d y\n%9s  ATOM=%d z\n",$4,v,$5,v,$6,v++)}'  \
       |  sed -e"s/-/ /g"                             \
       |  sort -n                                     \
       |  tail -1
   echo ""
fi
